/*
   ; Test and Braitenberg obstacle avoidance using MUST00a ultrasonic sensors
   ; ========================================================================
   ;
   ;-------------------------------------------------------------------------
   ; Author:    Skye Legon  27/02/01   (slegon@k-team.com)
   ; Modifs:
   ;
   ; This program reads 6 ultrasonic distance sensors and reports their
   ; distance (calibrated such that 0v -> 4v corresponds to 150mm -> 3000mm)
   ; or performs a simple Braitenberg-type obstacle avoidance.
   ;
   ;-------------------------------------------------------------------------
 */

#define BRAITENBERG 1  // set this to 1 to compile the Braitenberg version

#include 	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

#define MAX_DIST 3000  // max distance (mm) of sonar
#define MIN_DIST  150  // min distance (mm) of sonar
#define MEAN_SPEED 50  // mean speed of Koala
#define INV_GAIN 	100  // Braitenberg inverse gain value 

int32 _dist[6];

int32 SonarDistance(uint32 sensor)
{
	int32 value;

	// note sensor 0 is analog input 4 etc.
	if ((value = sens_get_ana_value(sensor+4)) < 0) return value;

	return (value * (MAX_DIST - MIN_DIST) + (MIN_DIST * 1023)) / 1023;
}

void procUpdateSonar()
{
	uint32 i;

	for (;;)
	{
		for (i=0; i<6; i++)
		{
			_dist[i] = SonarDistance(i);
		}
		tim_suspend_task(100);
	}
}

#if BRAITENBERG
void procMainTask()
{
	// simple Braitenberg obstacle avoidance (moves!)
	uint32 i;
	int32 speed[2];
	int32 deltaSpeed;
	int32 gainSign[6] = {1,1,1,-1,-1,-1};

	printf("Using sonar sensors to avoid obstacles...\r\n");
	printf("  0    1    2    3    4    5   L   R \r\n");
	printf("----+----+----+----+----+----+---+---\r\n");
	for (;;)
	{
		deltaSpeed = 0;
		for (i=0; i<6; i++)
		{
			printf("%4ld ", _dist[i]);
			deltaSpeed += gainSign[i] * (MAX_DIST - _dist[i]);
		}
		speed[0] = MEAN_SPEED + (deltaSpeed/INV_GAIN);
		speed[1] = MEAN_SPEED - (deltaSpeed/INV_GAIN);
		printf("%3ld ", speed[0]);
		printf("%3ld ", speed[1]);
		printf("                                         \r");
		fflush(stdout);
		mot_new_speed_2m(speed[1], speed[0]);
		tim_suspend_task(100);
	}
}
#else
void procMainTask()
{
	// just print the sonar values (don't move!)
	uint32 i;

	printf("Testing sonar sensors...\r\n");
	printf("  0    1    2    3    4    5 \r\n");
	printf("----+----+----+----+----+----\r\n");
	for (;;)
	{
		for (i=0; i<6; i++)
		{
			printf("%4ld ", _dist[i]);
		}
		printf("                                \r");
		fflush(stdout);
		tim_suspend_task(100);
	}
}
#endif

int main()
{
	sens_reset();
	mot_reset();

	install_task("procUpdateSonar\r\n", 800, procUpdateSonar);
	install_task("procMainTask\r\n", 800, procMainTask);

	return 0;
}
